import 'dart:async';
import 'dart:convert';

import 'package:flutter/services.dart';
import 'package:flutter_nfc_kit/flutter_nfc_kit.dart';
import 'package:prepaid_lib_flutter/ApduResponse.dart';

class UnikLibFlutter {
  static const MethodChannel _channel = const MethodChannel('unik_lib_flutter');

  static Future<String> get platformVersion async {
    final String version = await _channel.invokeMethod('getPlatformVersion');
    return version;
  }

  static void initUnikLib() async {
    await _channel.invokeMethod(
        'initUnikLib', <String, dynamic>{'mid': '49df5f81d59fae23ec6f542372fc9497', 'env': 1});
  }

  static Future<bool> getCardInfo(
      List<String> cardUid,
      List<String> cardNumber,
      List<String> balance,
      List<String> bankName,
      List<String> cardAttr,
      List<String> cardInfo) async {
    bool res = false;
    String apduClSelectApp = await _channel.invokeMethod('apduIsMyCard');
    String apduClCardInfo = await _channel.invokeMethod('apduCardInfo');
    String apduClBalance = await _channel.invokeMethod('apduBalance');
    String apduClCardAttr = await _channel.invokeMethod('apduCardAttr');

    try {
      NFCTag tag = await FlutterNfcKit.poll();
      cardUid[0] = tag.id;

      var dataSelect = jsonDecode(apduClSelectApp);
      var dataBalance = jsonDecode(apduClBalance);
      var dataCardAttr = jsonDecode(apduClCardAttr);

      List<ApduResponse> listData = dataSelect
          .map<ApduResponse>((result) => new ApduResponse.fromJson(result))
          .toList();
      List<ApduResponse> listApduBalance = dataBalance
          .map<ApduResponse>((result) => new ApduResponse.fromJson(result))
          .toList();
      List<ApduResponse> listApduCardAttr = dataCardAttr
          .map<ApduResponse>((result) => new ApduResponse.fromJson(result))
          .toList();

      for (ApduResponse s in listData) {
        String selectApp = await FlutterNfcKit.transceive(s.apdu);
        if (selectApp == "9000") {
          bankName[0] = s.bankType;
          print("bank name ${bankName[0]}");
        }
      }

      for (ApduResponse b in listApduBalance) {
        if (b.bankType == bankName[0]) {
          String apduBalance = await FlutterNfcKit.transceive(b.apdu);
          String rApduCardAttr = "";

          if (bankName[0] == "MANDIRI") {
            cardInfo[0] = await FlutterNfcKit.transceive(apduClCardInfo);
            for (ApduResponse apduCardAttr in listApduCardAttr) {
              if (apduCardAttr.bankType == bankName[0])
                rApduCardAttr =
                    await FlutterNfcKit.transceive(apduCardAttr.apdu);
            }

            String result = await _channel.invokeMethod(
                'parseBalance', <String, dynamic>{
              'apduBalance': "$apduBalance,${cardInfo[0]}",
              'bankType': bankName[0]
            });
            print("result $result");
            List<String> items = result.split(",");
            balance[0] = items[0];
            cardNumber[0] = items[1];
            cardAttr[0] = rApduCardAttr;
          } else {

            for (ApduResponse apduCardAttr in listApduCardAttr) {
              if (apduCardAttr.bankType == bankName[0])
                rApduCardAttr =
                    await FlutterNfcKit.transceive(apduCardAttr.apdu);
            }
            String composeBni = await _channel.invokeMethod('composeBNI',
                <String, dynamic>{'apduPurse': apduBalance + rApduCardAttr});

            String result = await _channel.invokeMethod(
                'parseBalance', <String, dynamic>{
              'apduBalance': apduBalance,
              'bankType': bankName[0]
            });
            print("result $result");
            List<String> items = result.split(",");
            balance[0] = items[0];
            cardNumber[0] = items[1];
            cardInfo[0] = composeBni ?? "0";
          }

          print("cardNumber = ${cardNumber[0]}, balance ${balance[0]}");

          res = true;
        }
      }
    } catch (error) {
      res = false;
    }
    return res;
  }

  static Future<bool> updateBalance(
    List<String> status,
    List<String> cardNumber,
    List<String> balance,
    List<String> bankName,
  ) async {
    bool result = false;
    List<String> cardUid = List(1);
    List<String> cardAttr = List(1);
    List<String> cardInfo = List(1);
    String rApduCardAttr = "";

    try {
      bool isSuccess = await UnikLibFlutter.getCardInfo(
          cardUid, cardNumber, balance, bankName, cardAttr, cardInfo);

      if (isSuccess) {
        if (await doReversalIfExist(cardNumber[0], "", "")) {
          String resUpdate =
              await _channel.invokeMethod('updateBalance', <String, dynamic>{
            'cardNumber': cardNumber[0],
            'cardUid': cardUid[0],
            'bankType': bankName[0],
            'balance': balance[0],
            'cardAttr': cardAttr[0] ?? "0",
            'respCardInfo': cardInfo[0] ?? "0"
          }).onError((error, stackTrace) {
            status[0] = error.toString();
          });

          List<String> items = resUpdate.toString().split(",");

          if (items[0] != "400") {
            // todo confirm update
            String rApdu = await FlutterNfcKit.transceive(items[1]);
            result = await confirmUpdate(
                cardUid[0], bankName[0], cardNumber[0], rApdu, status);
            print("result confirm on update $result");
          } else {
            status[0] = items[1];
            result = true;
          }
        }
      }
    } catch (error) {
      status[0] = "Something went wrong";
    }

    return result;
  }

  static Future<bool> doReversalIfExist(
      String cardNumber, String samData, String mode) async {
    var revResult = await _channel.invokeMethod(
        "checkingReversal", <String, dynamic>{
      'cardNumber': cardNumber,
      'samData': samData,
      'mode': mode
    });

    if (revResult == "") {
      return true;
    } else {
      String apdu = await FlutterNfcKit.transceive(revResult.toString());
      return doReversalIfExist(cardNumber, apdu, "REVERSAL_LOOP");
    }
  }

  static Future<bool> confirmUpdate(String cardUid, String bankType,
      String cardNumber, String samData, List<String> status) async {

    String respConfirm =
        await _channel.invokeMethod('confirm_update', <String, dynamic>{
      'cardUid': cardUid,
      'bankType': bankType,
      'cardNumber': cardNumber,
      'samData': samData
    });

    List<String> items = respConfirm.toString().split(",");
    if (items[0] == "") {
      print("respConfirm ${items[1]}");
      status[0] = items[1];
      return true;
    } else {
      String apdu = await FlutterNfcKit.transceive(items[0]);
      return confirmUpdate(cardUid, bankType, cardNumber, apdu, status);
    }
  }
}
